static XmlElement CreateWrappedSignature (AsymmetricAlgorithm signingKey, XmlElement contentToSign) {
    // Create the SignedXml message
    SignedXml signedXml = new SignedXml();
    signedXml.SigningKey = signingKey;
    // Create a DataObject to encapsulate the to-be-signed content
    // We use the string "ID1" as the internal reference ID for the object
    // Second and third arguments are null because we don't wish to specify
    // the optional MIME-Type and Encoding attributes.
    DataObject dataObject = new DataObject("ID1", null, null, contentToSign);
    // Add the new DataObject to the SignedXml object so it'll be included
    // in the output signature
    signedXml.AddObject(dataObject);
    // Create a Reference to point to the to-be-signed content
    Reference reference = new Reference();
    // Use a URI fragment to point to the same Id we used in the DataObject
    reference.Uri = "#ID1";
    // Add the Reference to the SignedXml message
    signedXml.AddReference(reference);
    // Compute the XML Digital Signature
    signedXml.ComputeSignature();
    // Return the XmlElement containing the XMLDSIG signature
    return signedXml.GetXml();
}
