static XmlElement CreateDetachedSignature(AsymmetricAlgorithm signingKey, String uri) {
    // Create the SignedXml message
    SignedXml signedXml = new SignedXml();
    signedXml.SigningKey = signingKey;
    // Create a Reference to point to the to-be-signed content
    Reference reference = new Reference(uri);
    // Create a Base64 transform: the input content retrieved from the
    // uri should be Base64-decoded before hashing
    Transform base64 = new XmlDsigBase64Transform();
    // Add the transform to the Reference object
    reference.AddTransform(base64);
    // Add the Reference to the SignedXml message
    signedXml.AddReference(reference);
    // Create a KeyInfo element to hold key-related information
    KeyInfo keyInfo = new KeyInfo();
    // Add an appropriate KeyValue sub-element
    if (signingKey is RSA) {
        keyInfo.AddClause(new RSAKeyValue((RSA) signingKey));
    } else if (signingKey is DSA) {
        keyInfo.AddClause(new DSAKeyValue((DSA) signingKey));
    }
    // Assign the KeyInfo object
    signedXml.KeyInfo = keyInfo;
    // Compute the XML Digital Signature
    signedXml.ComputeSignature();
    // Return the XmlElement containing the XMLDSIG signature
    return signedXml.GetXml();
}
