using System;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;
using System.IO;
using System.Text;
using System.Xml;

// Enveloped signature example

public class RSAEnvelopedSignature {

    static XmlElement CreateEnvelopedSignature(AsymmetricAlgorithm signingKey, XmlElement signatureParent) {
        // Create the SignedXml message
        SignedXml signedXml = new SignedXml(signatureParent);
        signedXml.SigningKey = signingKey;
        // Create a Reference to point to the to-be-signed content
        Reference reference = new Reference();
        // Use an empty-string URI to indicate that the signed content
        // is the document containing the Signature element
        reference.Uri = "";
        // Create an enveloped signature transform to reference the entire
        // document minus the Signature we're about to embed
        XmlDsigEnvelopedSignatureTransform envSigTransform = new XmlDsigEnvelopedSignatureTransform();
        // Add the transform to the Reference object
        reference.AddTransform(envSigTransform);
        // Add the Reference to the SignedXml message
        signedXml.AddReference(reference);
        // Create a KeyInfo element to hold key-related information
        KeyInfo keyInfo = new KeyInfo();
        // Add an appropriate KeyValue sub-element
        if (signingKey is RSA) {
            keyInfo.AddClause(new RSAKeyValue((RSA) signingKey));
        } else if (signingKey is DSA) {
            keyInfo.AddClause(new DSAKeyValue((DSA) signingKey));
        }
        // Assign the KeyInfo object
        signedXml.KeyInfo = keyInfo;
        // Compute the XML Digital Signature
        signedXml.ComputeSignature();
        // Return the XmlElement containing the XMLDSIG signature
        return signedXml.GetXml();
    }

    public static void Main() 
    {
        // Create a random RSA key
        RSACryptoServiceProvider rsa;
        rsa = new RSACryptoServiceProvider();
        // Create a new XmlDocument with the to-be-signed content
        XmlDocument doc = new XmlDocument();
        doc.LoadXml("<MyElement xmlns=\"\"><MyFirstChild>First Child's Text</MyFirstChild><MySecondChild>Second Child's Text</MySecondChild></MyElement>");
        // Get the root document element
        XmlElement elem = (XmlElement) doc.DocumentElement;
        // Create an enveloped signature using the root document element as
        // the parent of the signature
        XmlElement xmlDigitalSignature = CreateEnvelopedSignature(rsa, elem);
        // Add the computed Signature element to the root document element
        elem.AppendChild(xmlDigitalSignature);
        // Create an XmlTextWriter to write out the XML content to the Console
        XmlTextWriter xmltw;
        xmltw = new XmlTextWriter(Console.Out);
        // Write out the document
        doc.WriteTo(xmltw);
        // Close the XmlTextWriter
        xmltw.Close();
    }
}
