using System;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;
using System.IO;
using System.Xml;
using System.Text;

public class VerifySignature {

  static void Main(String[] args) {
    // Create a new XmlDocument object
    XmlDocument xmlDocument = new XmlDocument();
    // Tell the XmlDocument to preserve white space in the input
    xmlDocument.PreserveWhitespace = true;
    // Load the contents of the file into the XmlDocument
    xmlDocument.Load(new XmlTextReader(args[0]));

    // Our XPath query is going to look for a Signature tag in the
    // XMLDSIG namespace, so we have to initialize an XmlNamespaceManager
    // and tell it about the URI that identifies the XMLDSIG namespace
    // The next two lines of code do that.
    XmlNamespaceManager nsm = new XmlNamespaceManager(xmlDocument.NameTable);
    nsm.AddNamespace("ds", SignedXml.XmlDsigNamespaceUrl);
    // Now call the XPath query method on XmlDocument
    // This will return the first XmlNode that matches the
    // "//ds:Signature" query. That query will search the entire XML
    // tree underneath the document root looking for an element with
    // tag name "Signature" in the namespace bound to "ds" in the
    // XmlNamespaceManager.
    XmlNode node = xmlDocument.SelectSingleNode("//ds:Signature", nsm);

    // Create a new SignedXml object
    SignedXml signedXml = new SignedXml();
    // Load in the Signature element
    signedXml.LoadXml((XmlElement)node);

    AsymmetricAlgorithm key;
    // Use CheckSignatureReturningKey so we get a handle
    // back on the key object that was discovered to
    // be the validation key
    if (signedXml.CheckSignatureReturningKey(out key)) {
      Console.WriteLine("Signature check OK");
      Console.WriteLine(key.ToXmlString(false));
    } else {
      Console.WriteLine("Signature check FAILED");
    }
    return;
  }
}
