using System;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;
using System.IO;
using System.Xml;
using System.Text;
using System.Collections;
using System.Runtime.InteropServices;

public class MySignedXml : SignedXml {

  IEnumerator m_keyInfoEnum = null;

  public MySignedXml() : base() {
  }

  [DllImport("advapi32.dll", EntryPoint="CryptAcquireContext", CharSet=CharSet.Unicode)]
  public static extern bool CryptAcquireContext(IntPtr phProv, String pszContainer, String pszProvider, int dwProvType, int dwFlags);

  // Utility method for checking whether a specific key container
  // is present in the default Type 1 CSP
  private static bool CheckForPresenceOfKeyContainer(String keyContainerName) {
    IntPtr phProv = new IntPtr(0);
    bool result = CryptAcquireContext(phProv, keyContainerName, null, 1, 0);
    return result;
  }

  protected override AsymmetricAlgorithm GetPublicKey() {
    RSAKeyValue tempRSA;
    DSAKeyValue tempDSA;
    KeyInfoName tempName;

    if (m_keyInfoEnum == null)
      m_keyInfoEnum = KeyInfo.GetEnumerator();

    // In our implementation, we move to the next KeyInfo clause 
    // which is an RSAKeyValue or a DSAKeyValue
    while (m_keyInfoEnum.MoveNext()) {
      tempRSA = m_keyInfoEnum.Current as RSAKeyValue;
      if (tempRSA != null) return(tempRSA.Key);
      tempDSA = m_keyInfoEnum.Current as DSAKeyValue;
      if (tempDSA != null) return(tempDSA.Key);
      tempName = m_keyInfoEnum.Current as KeyInfoName;
      if (tempName != null) {
        // Use the string value of the KeyInfoName as the public key container
        // Check to see if the container is present
        // We assume the key is an RSA key in this example
        if (CheckForPresenceOfKeyContainer(tempName.Value)) {
          // Create a new CspParameters referencing the key container
          CspParameters cspParams = new CspParameters();
          cspParams.KeyContainerName = tempName.Value;
          // Create an RSACryptoServiceProvider referencing the key container
          RSACryptoServiceProvider rsa = new RSACryptoServiceProvider(cspParams);
          return(rsa);
        }
      }
    }
    return(null);
  }
}
